/**
 * Handles database connections and queries
 */
package org.penguindreams.bmcb.data;



import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.penguindreams.bmcb.BMCBMain;
import org.penguindreams.bmcb.util.ApplicationLogger;
import org.penguindreams.bmcb.util.InvalidConfigException;

public class DataBase {

	//private String host, user, pass, db;
	private int port;
	
	private Connection conn;
	
	private Logger log;
	
	//They are necessary for DB_OPTS bellow 
	// They must be set manually and seperately in the constructor
	private String host, user,pass,db;
	
	private final String[][] DB_OPTS = {
		 {"db_host", host},{"db_user",user},{"db_password",pass},{"db_database",db}
	};
		
	public DataBase() {
		
		log = ApplicationLogger.getLogger(this.getClass());
		
		//load database options from config file object initialized in Main class
		log.debug("Loading Database Options");
		try {
			
			for(String[] s : DB_OPTS) {
				s[1] = BMCBMain.config.getConfigValue(s[0]);
			}
			
			try {
				this.port = Integer.valueOf(BMCBMain.config.getConfigValue("db_port"));
			} catch (NumberFormatException e1) {
				log.fatal("Invalid Database Port Number" + BMCBMain.config.getConfigValue("db_port"));
			}
			
			//I have to do this here. The array should reference 
			//the private variables but since they're references
			//and not pointers, you can't alter the first by changing 
			//the later unless the object has all ready been instantiated
			host = DB_OPTS[0][1];
			user = DB_OPTS[1][1];
			pass = DB_OPTS[2][1];
			db   = DB_OPTS[3][1];

		} catch (InvalidConfigException e1) {
			log.fatal("Missing Database Config Option");
			log.fatal(e1.getMessage());
			System.exit(3);
		}
		log.debug("Database Options Loadad");
		
		
		try {
			log.info("Loading MySQL Driver");
            Class.forName("com.mysql.jdbc.Driver");
    	} catch (ClassNotFoundException e) {
            //throw new Exception("Could not load MySQL Driver");
    		log.fatal("Could load MySQL Driver");
    		System.exit(2);		
    	}

		
		try { 
			log.info("Attempting to Establish SQL connection with " + host + ":" + port);
			conn = DriverManager.getConnection("jdbc:mysql://"+host+":"+port+"/"+db , user , pass);
			log.info("Database Connection Established");
		} catch (SQLException ex) { 
			// handle any errors 
			log.error("SQLException: " + ex.getMessage()); 
			log.error("SQLState: " + ex.getSQLState()); 
			log.error("VendorError: " + ex.getErrorCode()); 
		} 

	}
	
	public DataSet getDataSet(String name) {
		//throw away / reusable variables
		PreparedStatement ps;
		ResultSet rs;
		String[] retval;

		int setID = getSetID(name);
		String logName = "\"" + name + "\" (" + setID + ")";
		
		try {
			//get a count
			ps = conn.prepareStatement("SELECT COUNT(*) as count FROM setdata WHERE set_id=?");
			ps.setInt(1, setID);
			rs = ps.executeQuery();
			if(rs.next()) {
				int count = rs.getInt("count");
				log.info("Set " + logName + " contains " + count + " records");
				retval = new String[count];
			}
			else {
				log.error("Could not pull count from " + logName);
				return null;
			}
			
			ps = conn.prepareStatement("SELECT * FROM setdata WHERE set_id=?");
			ps.setInt(1, setID);
			rs = ps.executeQuery();
			int i=0;
			while(rs.next()) {
				retval[i] = rs.getString("data");
				i++;
			}
			log.info("Successfully pulled " + logName);
			return new DataSet(name,retval);
			
		} catch (SQLException e) {
			log.error("An SQL Exception Occured when trying to pull the set " + logName);
			return null;
		}
		
	}
	
	public boolean writeDataSet(DataSet set) {
		
		//throw away / reusable variables
		PreparedStatement ps;
		ResultSet rs;
		int i = 0;
		int setID = -1;
		
		try {
			

			
			//sanity check
			ps = conn.prepareStatement("SELECT COUNT(*) AS count FROM sets WHERE name=?");
			ps.setString(1, set.getName());
			ps.execute();
			rs = ps.getResultSet();
			if(rs.next()) {
				if(rs.getInt("count") != 0) {
					log.error("Set "+set.getName()+" all ready exists. You must delete the set if you wish to recreate it");
					return false;
				}
			}
			else {
				log.warn("No results from duplicate set check. Duplicate set may have been created");
			}
		
			//create set set.getName()
			ps = conn.prepareStatement("INSERT INTO sets (name) VALUES(?)",Statement.RETURN_GENERATED_KEYS);
			ps.setString(1, set.getName());

			if(   (i = ps.executeUpdate()) != 1) {
				log.warn("Creating set set.getName() for " + set.getName() + "did not return one row updated! (rows: "+i+")");
			}
			
			//get generated key for set ID
			rs = ps.getGeneratedKeys();
			if(rs.next()) {
				setID = rs.getInt(1);
				log.debug("Set ID for " + set.getName() + " is " + setID);
			}
			else {
				log.error("Creating set " + set.getName() + " did not generate ID");
				return false;
			}
			

			//populate actual dataset
			ps = conn.prepareStatement("INSERT INTO setdata (set_id,data) VALUES(?,?)");			
			
			for(i=0; i < set.getSize(); i++) {

				ps.setInt(1,setID);
				ps.setString(2, set.get(i));
				try{
					ps.execute();
				}
				catch(SQLException e){
					log.error("Failed to insert \"" + set.get(i) + "\" into set " + set.getName() + " (" + setID + ")" );
				}
			}
			
			
		} catch (SQLException e) {
			log.error("SQL Error Encountered while creating dataset " + set.getName() + " : " + e.getMessage());
			return false;
		}
		
		//nothing failed
		log.info("Set " + set.getName() + " ("+setID+") successfully created");
		return true;
	}
	
	public void deleteDataSet(String name) {

		//throw away / reusable variables
		PreparedStatement ps;
		int setID = getSetID(name);
		
		try {
			//delete the set label first
			ps = conn.prepareStatement("DELETE FROM sets WHERE id=?");
			ps.setInt(1, setID);
			try{
				ps.execute();
				log.info("Deleted set label " + name);
			}
			catch(SQLException e) {
				log.error("Unable to delete set label " + name);
			}
			
			//delete all the set data
			ps = conn.prepareStatement("DELETE FROM setdata WHERE set_id=?");
			ps.setInt(1, setID);
			int rows = ps.executeUpdate();
			log.info(rows + " records for dataset \"" + name + "\" ("+ setID +") deleted" );
			
		} catch (SQLException e) {
			log.error("SQL Error occurred when attempting to delete dataset " + name);
			log.error(e.getMessage());
		}
	}
	
	public String[] getDataSets() {
		
		PreparedStatement ps;
		ArrayList<String> retval = new ArrayList<String>();
		
		try {
			ps = conn.prepareStatement("SELECT name FROM sets");
			ResultSet rs = ps.executeQuery();
			while(rs.next()) {
				retval.add(rs.getString("name"));
			}
			return retval.toArray(new String[0]);

		}
		catch(SQLException e) {
			log.error("SQL Exception occured when attempting to get list of all datasets");
			log.error(e.getMessage());
			return null;
		}
	}
	
	private int getSetID(String name) {
		//throw away / reusable variables
		PreparedStatement ps;
		ResultSet rs;
		
		try {
			ps = conn.prepareStatement("SELECT id FROM sets WHERE name=?");
			ps.setString(1, name);
			rs = ps.executeQuery();
			if(rs.next()) {
				return rs.getInt(1);
			}
			else {
				log.warn("Dataset " + name + " was not found");
				return -1;
			}
		}
		catch(SQLException e) {
			log.error("SQL Error occurred when attempting to retrieve ID for dataset " + name );
			log.error(e.getMessage());
			return -1;
		}
	}
	
	
}
