/**
 * The file contains static Image loading and manipulation functions.
 */
package org.penguindreams.bmcb.util;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;


import net.sourceforge.jiu.codecs.CodecMode;
import net.sourceforge.jiu.codecs.PNMCodec;
import net.sourceforge.jiu.gui.awt.ImageCreator;
import net.sourceforge.jiu.ops.MissingParameterException;
import net.sourceforge.jiu.ops.OperationFailedException;

import org.apache.log4j.Logger;
import org.penguindreams.bmcb.BMCBMain;
import org.penguindreams.bmcb.data.DataSet;



public class ImageLoader {

	/**
	 * All extensions/file types are standardized to JPEGs
	 */
	public static final String IMG_EXTENTION = "jpeg";
	
	public static Logger log = ApplicationLogger.getLogger(ImageLoader.class);
	
	/**
	 * loads an BufferedImage from a File object
	 * @param f File to load into BufferedImage
	 * @return BufferedImage representing the file
	 * @throws IOException if an error occurred finding or loading the file
	 */
	public static BufferedImage loadImageFromFile(File f) throws IOException {
			return ImageIO.read(f);
	}
	
	/**
	 * draws red axes on image.
	 * Used for drawing axes returned by Segmentator for testing purposes.
	 * @param img BufferedImage to draw upon
	 * @param array of vertical axes locations starting where x=0 is the left most column in image 
	 * @throws IOException
	 */
	public static void drawAxes(BufferedImage img, int[] axes) throws IOException {
		
		for(int ax : axes) {
			Graphics g = img.getGraphics();
			g.setColor(Color.RED);
			g.drawLine(ax, 0, ax, img.getHeight());
		}
		
		//return img;
	}
	
	/**
	 * converts an Image into a BufferedImage.
	 * New Buffered Image is a copy of the Image. The original Image object is not modified.
	 * @param im Image object to convert
	 * @return BufferedImage containing copy of original Image object
	 */
	 public static BufferedImage createBufferedImage(Image im)
	 {
	    BufferedImage bi = new BufferedImage(im.getWidth(null),im.getHeight(null),BufferedImage.TYPE_INT_RGB);
	    Graphics bg = bi.getGraphics();
	    bg.drawImage(im, 0, 0, null);
	    bg.dispose();
	    return bi;
	 }
	
	/**
	 * constructs a path to a particular element in a dataset.
	 * The path is determined using the IMG_EXTENTION constant and 
	 * information from the BMCBMain.conf object
	 * @param d DataSet where element resides
	 * @param f Actual element
	 * @return File object with absolute path to image file
	 */
	public static File constructFilePath(DataSet d, String f) {
		try {
			return new File(new File(BMCBMain.config.getConfigValue("cp_setdirs"),d.getName()),f+"."+IMG_EXTENTION);
		} catch (InvalidConfigException e) {
			log.fatal("Missing Config Attribute for Set Directory when trying to construct path");
			System.exit(10);
			return null;
		}
	}
	
	
	/**
	 * Writes an Image out to a PNM File
	 * @param image Image to write to file
	 * @param out File to output
	 * @throws IOException
	 * @throws MissingParameterException
	 * @throws OperationFailedException
	 */
	public static void saveToPNM(Image image, File out) throws IOException, MissingParameterException, OperationFailedException {
		PNMCodec codec = new PNMCodec();
		codec.setFile(out, CodecMode.SAVE);
		codec.setImage(ImageCreator.convertImageToRGB24Image(image)); // image to be saved
		codec.process();
		codec.close();
	}
	
}





